﻿using System;
using System.CodeDom.Compiler;
using System.Reflection;
using System.Text;
using Microsoft.CSharp;
using GrapeCity.Competition.Maze;

namespace GrapeCity.Competition.Maze.Debuger
{
    public abstract class ScriptCompilerBase
    {

        protected abstract CodeDomProvider CodeDomProvider { get; }

        private static readonly string[] ReferencedAssemblies = new string[]
                                                                {
                                                                    "System.dll", 
                                                                    "System.Data.dll", 
                                                                    "System.Xml.dll", 
                                                                    "MazeLib.dll"
                                                                };

        #region IScriptCompiler Members

        public Assembly Compile(string sourceCode, out string compileError)
        {
            CompilerResults results;
            try
            {
                CompilerParameters parameters = new CompilerParameters();

                parameters.GenerateInMemory = true;
                parameters.TreatWarningsAsErrors = false;
                parameters.WarningLevel = 1;
                parameters.ReferencedAssemblies.AddRange(ReferencedAssemblies);

                results = CodeDomProvider.CompileAssemblyFromSource(parameters, sourceCode);
                compileError = GenerateOutputText(results);
            }
            catch (Exception exception)
            {
                compileError = exception.Message;
                return null;
            }

            if (results.Errors.Count == 0)
            {
                return results.CompiledAssembly;
            }

            return null;
        }

        public bool Build(string sourceCode, string outputName, out string compileError)
        {
            CompilerResults results;
            try
            {
                CompilerParameters parameters = new CompilerParameters();
                parameters.GenerateExecutable = false;
                parameters.OutputAssembly = outputName;
                parameters.GenerateInMemory = false;
                parameters.TreatWarningsAsErrors = false;
                parameters.ReferencedAssemblies.AddRange(ReferencedAssemblies);

                results = (new CSharpCodeProvider()).CompileAssemblyFromSource(parameters, sourceCode);
                compileError = GenerateOutputText(results);
            
            }
            catch (Exception exception)
            {
                compileError = exception.Message;
                return false;
            }

            if (results.Errors.Count == 0)
            {
                return true;
            }

            return false;
        }

        #endregion

        private static string GenerateOutputText(CompilerResults results)
        {
            StringBuilder errorBuilder = new StringBuilder();

            if(results.Errors.Count > 0)
            {
                foreach (CompilerError error in results.Errors)
                {
                    errorBuilder.Append(string.Format("Line {0},Col {1},Error {2} - {3} \n", error.Line, error.Column, error.ErrorNumber, error.ErrorText));
                }
                return errorBuilder.ToString();
            }

            errorBuilder.Append(string.Format("Compile complete -- {0} errors", results.Errors.Count));

            return errorBuilder.ToString();
        }


    }
}
