﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Reflection;
using System.Text;
using System.Threading;
using System.Windows.Forms;

namespace GrapeCity.Competition.Maze.Debuger
{
    public partial class ScriptEditorDialog : Form
    {

        private Assembly compiledAssembly = null;


        public ScriptEditorDialog()
        {
            InitializeComponent();

            this.languageToolStripComboBox.Items.AddRange(SearchCompilers());
            if(this.languageToolStripComboBox.Items.Count > 0)
            {
                this.languageToolStripComboBox.SelectedIndex = 0;
            }
        }

        public Assembly CompiledAssembly
        {
            get { return compiledAssembly; }
        }

        private static NamedType[] SearchCompilers()
        {
            Assembly container = typeof(IScriptCompiler).Assembly;

            List<NamedType> compilers = new List<NamedType>();
            foreach (Type type in container.GetTypes())
            {
                if (type.GetInterface("IScriptCompiler") == typeof(IScriptCompiler))
                {
                    compilers.Add(new NamedType(type));
                }
            }

            return compilers.ToArray();
        }

        private string UserScript
        {
            get
            {
                if(string.IsNullOrEmpty(scriptTextBox.Text))
                {
                    return null;
                }
                else
                {
                    return scriptTextBox.Text;
                }
            }
        }

        private IScriptCompiler UserSelectedCompiler
        {
            get
            {
                NamedType namedType = languageToolStripComboBox.SelectedItem as NamedType;

                if (namedType != null)
                {
                    try
                    {
                        IScriptCompiler compiler = Activator.CreateInstance(namedType.Type) as IScriptCompiler;
                        return compiler;
                    }
                    catch 
                    {
                        return null; 
                    }
                }

                return null;
            }
        }

        private string UserSelectedAssemblyName
        {
            get
            {
                if (saveFileDialog.ShowDialog() == DialogResult.OK)
                {
                    return saveFileDialog.FileName;
                }
                else
                {
                    return null;
                }
            }
        }

        private void ShowCompileError(string error)
        {
            MessageBox.Show(this, error, "Compile Result", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }

        private void runToolStripButton_Click(object sender, EventArgs e)
        {
            string scriptErrors;
            IScriptCompiler com = UserSelectedCompiler;
            string script = UserScript;

            Assembly assembly = com.Compile(script, out scriptErrors);
            if(assembly != null)
            {
                this.compiledAssembly = assembly;
                this.DialogResult = DialogResult.OK;
                this.Close();
            }
            else
            {
                ShowCompileError(scriptErrors);
            }
        }

        private void saveToolStripButton_Click(object sender, EventArgs e)
        {
            string assemblyFile;

            if (saveFileDialog.ShowDialog() == DialogResult.OK)
            {
                assemblyFile = saveFileDialog.FileName;
            }
            else
            {
                assemblyFile = null;
            }

            if (string.IsNullOrEmpty(assemblyFile)) return;

            string scriptErrors;
            IScriptCompiler com = UserSelectedCompiler;

            if (com == null)
            {
                ShowUserCompilerMustSelected();
                return;
            }

            string script = UserScript;
            if (script == null)
            {
                ShowUserScriptCannotEmpty();
                return;
            }

            if (com.Build(script, assemblyFile, out scriptErrors))
            {
                ShowUserBuildComplete();
            }
            else
            {
                ShowCompileError(scriptErrors);
            }
        }

        private void ShowUserScriptCannotEmpty()
        {
            MessageBox.Show(this, "Script can not empty.", "Compile Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }

        private void ShowUserCompilerMustSelected()
        {
            MessageBox.Show(this, "You must select a language.", "Compile Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }

        private void ShowUserBuildComplete()
        {
            MessageBox.Show(this, "Build success.", "Compile Result", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        private void compileToolStripButton_Click(object sender, EventArgs e)
        {
            string scriptErrors;
            IScriptCompiler com = UserSelectedCompiler;
            string script = UserScript;

            Assembly assembly = com.Compile(script, out scriptErrors);
            if (assembly != null)
            {
                ShowUserBuildComplete();
            }
            else
            {
                ShowCompileError(scriptErrors);
            }
        }
    }
}
