﻿using System;
using GrapeCity.Competition.Maze.Engine.Explorers;
using GrapeCity.Competition.Maze.Engine.Maps;

namespace GrapeCity.Competition.Maze.Engine.Elements
{
    #region The Door class
    /// <summary>
    ///   Represents the element indicate a door.
    /// </summary>
    [Serializable]
    public class Door : MazeElement
    {
        #region Instance Data
        /// <summary>
        ///   Indicates whether the door is locked.
        /// </summary>
        private bool m_locked = true;
        #endregion

        #region Constructor
        /// <summary>
        ///   Initialize the <see cref="Door"/> object.
        /// </summary>
        /// <param name="mazeMap">
        ///   A <see cref="IMazeMap"/> object indicates the maze map.
        /// </param>
        public Door(IMazeMap mazeMap)
            : base(mazeMap)
        {
        }
        #endregion

        #region Clone
        /// <summary>
        ///   Creates a new object that is a copy of the current instance.
        /// </summary>
        /// <returns>
        ///   A new object that is a copy of this instance.
        /// </returns>
        public override MazeElement Clone(IMazeMap map)
        {
            Door door = new Door(map);
            door.m_locked = this.m_locked;

            return door;
        }
        #endregion

        #region Properties
        /// <summary>
        ///   Gets or sets whether the door is locked.
        /// </summary>
        /// <value>
        ///   <b>true</b>, if the door is locked; otherwise, <b>false</b>.
        /// </value>
        public bool IsLocked
        {
            get
            {
                return this.m_locked;
            }
            set
            {
                this.m_locked = value;
            }
        }
        /// <summary>
        ///   Gets the information in the position.
        /// </summary>
        /// <value>
        ///   A <see cref="PositionInfo"/> value indicates the position information.
        /// </value>
        public override PositionInfo ElementInfo
        {
            get
            {
                return PositionInfo.Door;
            }
        }
        #endregion

        #region Methods
        /// <summary>
        ///   Open the door.
        /// </summary>
        public void Open()
        {
            this.m_locked = false;
        }
        /// <summary>
        ///   Close the door.
        /// </summary>
        public void Close()
        {
            this.m_locked = true;
        }

        /// <summary>
        ///   Occurs when an explorer try to enter the position.
        /// </summary>
        /// <param name="explorer">
        ///   An <see cref="IExplorer"/> object indicates the explorer.
        /// </param>
        /// <returns>
        ///   <b>true</b>, if the explorer can enter the position; otherwise, <b>false</b>.
        /// </returns>
        public override bool CanExplorerIn(IMazeExplorer explorer)
        {
            return !this.m_locked;
        }

        #endregion
    }
    #endregion
}