﻿using System;
using GrapeCity.Competition.Maze.Engine.Explorers;
using GrapeCity.Competition.Maze.Engine.Maps;

namespace GrapeCity.Competition.Maze.Engine.Elements
{
    #region The Entrance class
    /// <summary>
    ///   Represents the element indicate a entrance.
    /// </summary>
    [Serializable]
    public class Entrance : MazeElement
    {
        #region Constructor
        /// <summary>
        ///   Initialize the <see cref="Entrance"/> object.
        /// </summary>
        /// <param name="mazeMap">
        ///   A <see cref="IMazeMap"/> object indicates the maze map.
        /// </param>
        public Entrance(IMazeMap mazeMap)
            : base(mazeMap)
        {
        }
        #endregion

        #region Clone
        /// <summary>
        ///   Creates a new object that is a copy of the current instance.
        /// </summary>
        /// <returns>
        ///   A new object that is a copy of this instance.
        /// </returns>
        public override MazeElement Clone(IMazeMap map)
        {
            return new Entrance(map);
        }
        #endregion

        #region Properties
        /// <summary>
        ///   Gets the information in the position.
        /// </summary>
        /// <value>
        ///   A <see cref="PositionInfo"/> value indicates the position information.
        /// </value>
        public override PositionInfo ElementInfo
        {
            get
            {
                return PositionInfo.Entrance;
            }
        }
        #endregion

        #region Methods

        /// <summary>
        ///   Occurs when an explorer try to enter the position.
        /// </summary>
        /// <param name="explorer">
        ///   An <see cref="IExplorer"/> object indicates the explorer.
        /// </param>
        /// <returns>
        ///   <b>true</b>, if the explorer can enter the position; otherwise, <b>false</b>.
        /// </returns>
        public override bool CanExplorerIn(IMazeExplorer explorer)
        {
            return true;
        }

        #endregion
    }
    #endregion
}