﻿using System;
using GrapeCity.Competition.Maze;
using GrapeCity.Competition.Maze.Engine.Explorers;
using GrapeCity.Competition.Maze.Engine.Maps;

namespace GrapeCity.Competition.Maze.Engine.Elements
{
    #region The Transport class
    /// <summary>
    ///   Represents the element indicate a transport.
    /// </summary>
    [Serializable]
    public class Transport : MazeElement
    {
        private Position m_transportTo;

        #region Constructor
        /// <summary>
        ///   Initialize the <see cref="Transport"/> object.
        /// </summary>
        /// <param name="mazeMap">
        ///   A <see cref="IMazeMap"/> object indicates the maze map.
        /// </param>
        public Transport(IMazeMap mazeMap)
            : base(mazeMap)
        {
            m_transportTo = new Position(mazeMap.Entrance);
        }
        #endregion

        #region Clone
        /// <summary>
        ///   Creates a new object that is a copy of the current instance.
        /// </summary>
        /// <returns>
        ///   A new object that is a copy of this instance.
        /// </returns>
        public override MazeElement Clone(IMazeMap map)
        {
            Transport t = new Transport(map);
            t.TransportTo = this.TransportTo;
            return t;
        }
        #endregion

        #region Properties
        /// <summary>
        ///   Gets the information in the position.
        /// </summary>
        /// <value>
        ///   A <see cref="PositionInfo"/> value indicates the position information.
        /// </value>
        public override PositionInfo ElementInfo
        {
            get
            {
                return PositionInfo.Transport;
            }
        }

        /// <summary>
        /// Gets or sets the position for transport.
        /// </summary>
        /// <value>
        ///  A <see cref="Position"/> value indicates the transport position.
        /// </value>
        public Position TransportTo
        {
            get { return m_transportTo; }
            set { m_transportTo = value; }
        }

        #endregion

        #region Methods

        /// <summary>
        ///   Occurs when an explorer try to enter the position.
        /// </summary>
        /// <param name="explorer">
        ///   An <see cref="IExplorer"/> object indicates the explorer.
        /// </param>
        /// <returns>
        ///   <b>true</b>, if the explorer can enter the position; otherwise, <b>false</b>.
        /// </returns>
        public override bool CanExplorerIn(IMazeExplorer explorer)
        {
            return true;
        }

        /// <summary>
        ///   Occurs when a explorer stay at the position.
        /// </summary>
        /// <param name="explorer">
        ///   A <see cref="IExplorer"/> object indicates the explorer.
        /// </param> 
        public override void OnExplorerStay(IMazeExplorer explorer)
        {
            if(!MazeMap.IsValidPosition(TransportTo)) return;
            ((IMobile)explorer).X = TransportTo.X;
            ((IMobile)explorer).Y = TransportTo.Y;
        }
        #endregion
    }
    #endregion
}
