﻿using System;

namespace GrapeCity.Competition.Maze.Engine.Maps
{
    #region The IMazeMap interface

    /// <summary>
    ///   Represents the maze map.
    /// </summary>
    public interface IMazeMap : IMaze
    {
        /// <summary>
        ///   Check whether it is a valid position.
        /// </summary>
        /// <param name="position">
        ///   A <see cref="IPosition"/> indicates the specified position.
        /// </param>
        /// <returns>
        ///   <b>true</b>, if it is a valid position; otherwise, <b>false</b>.
        /// </returns>
        bool IsValidPosition(IPosition position);

        /// <summary>
        ///   Check whether the coordinate pair is a valid position.
        /// </summary>
        /// <param name="x">
        ///   The x-coordinate of the upper-left corner of the element to check. 
        /// </param>
        /// <param name="y">
        ///  The y-coordinate of the upper-left corner of the element to check. 
        /// </param>
        /// <returns>
        ///   <b>true</b>, if the coordinate pair is a valid position; otherwise, <b>false</b>.
        /// </returns>
        bool IsValidPosition(int x, int y);

        /// <summary>
        ///   Gets the element based specified position.
        /// </summary>
        /// <param name="position">
        ///   A <see cref="IPosition"/> value indicates the specified position.
        /// </param>
        /// <returns>
        ///   A <see cref="MazeElement"/> object indicates the element.
        /// </returns>
        MazeElement GetElement(IPosition position);

        /// <summary>
        ///   Gets the element specified by a coordinate pair.
        /// </summary>
        /// <param name="x">
        ///   The x-coordinate of the upper-left corner of the element to get. 
        /// </param>
        /// <param name="y">
        ///  The y-coordinate of the upper-left corner of the element to get. 
        /// </param>
        /// <returns>
        ///   A <see cref="MazeElement"/> object indicates the element.
        /// </returns>
        MazeElement GetElement(int x, int y);
    }
    #endregion
}
