﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Text;

namespace GrapeCity.Competition.Maze.Engine.Maps
{
    #region The MazeMapClient class

    /// <summary>
    ///   Represents the maze map editor.
    /// </summary>
    internal class MazeMapClient : IMaze
    {
        #region IMaze Members

        /// <summary>
        ///   Gets the maze name.
        /// </summary>
        /// <value>
        ///   A <b>string</b> value indicates the maze name.
        /// </value>
        string IMaze.Name
        {
            get { return mazeName; }
        }

        /// <summary>
        ///   Gets the width of maze.
        /// </summary>
        /// <value>
        ///   An <b>int</b> value indicates the maze width.
        /// </value>
        int IMaze.Width
        {
            get { return mazeWidth; }
        }

        /// <summary>
        ///   Gets the height of maze.
        /// </summary>
        /// <value>
        ///   An <b>int</b> value indicates the maze height.
        /// </value>
        int IMaze.Height
        {
            get { return mazeHeight; }
        }

        /// <summary>
        ///   Gets the entrance position of maze.
        /// </summary>
        /// <value>
        ///   An <see cref="IPosition"/> value indicates the entrance position of maze.
        /// </value>
        IPosition IMaze.Entrance
        {
            get { return mazeEntrance; }
        }

        /// <summary>
        ///   Gets the exit position of maze.
        /// </summary>
        /// <value>
        ///   An <see cref="IPosition"/> value indicates the exit position of maze.
        /// </value>
        IPosition IMaze.Exit
        {
            get { return mazeExit; }
        }

        #endregion

        #region Instance Data

        private readonly string mazeName;
        private readonly int mazeWidth;
        private readonly int mazeHeight;
        private readonly IPosition mazeEntrance;
        private readonly IPosition mazeExit;

        #endregion

        #region Constructor

        /// <summary>
        ///   Initialize the <see cref="MazeMapClient"/> object.
        /// </summary>
        /// <param name="maze"></param>
        internal MazeMapClient(IMaze maze)
        {
            Trace.Assert(maze != null);

            this.mazeName = string.Copy(maze.Name);
            this.mazeWidth = maze.Width;
            this.mazeHeight = maze.Height;
            this.mazeEntrance = new Position(maze.Entrance);
            this.mazeExit = new Position(maze.Exit);
        }

        #endregion
    }

    #endregion
}
