﻿using System;
using System.Diagnostics;

namespace GrapeCity.Competition.Maze.Engine
{

    #region The Thinker Class

    public class Thinker : IMazeAI
    {
        private readonly IMazeAI instance;
        private readonly Type thinkerType;
        private long elapsedTime;

        public Thinker(Type typeOfMazeAI)
        {
            if(typeOfMazeAI == null)
                throw new NullReferenceException("AI type is null.");

            try
            {
                this.instance = Activator.CreateInstance(typeOfMazeAI) as IMazeAI;

                if (instance == null)
                    throw new NullReferenceException("AI instance is null.");

                this.thinkerType = typeOfMazeAI;
            }
            catch(Exception e)
            {
                throw new TypeInitializationException(typeOfMazeAI.FullName, e);
            }
        }

        public string Name
        {
            get { return NameAttribute.GetName(thinkerType); }
        }

        public string Author
        {
            get { return AuthorAttribute.GetName(thinkerType); }
        }

        public string FullName
        {
            get { return thinkerType.AssemblyQualifiedName; }
        }

        public long ElapsedTime
        {
            get { return elapsedTime; }
        }

        public void Initialize()
        {
            instance.Initialize();
        }

        public void Terminate()
        {
            instance.Terminate();
        }

        public void Think(IMaze maze, IExplorer explorer)
        {
            Stopwatch watch = new Stopwatch();
            watch.Start();
            instance.Think(maze, explorer);
            watch.Stop();
            elapsedTime = watch.ElapsedMilliseconds;
        }

        public void Hear(string name, string message, object information)
        {
            instance.Hear(name, message, information);
        }
    }

    #endregion

}
