#region Copyright

/*
 * Project:		Agent for .NET Framework
 * File:		GrapeCity.Agent.RuntimeModel.Time.cs
 * Author:		ToxZhang
 * Create:		2007/08/03 03:52:42 PM
 *
 * FileId:		7995799c-810c-4485-bcb3-63b321f050ab
 * Type:		CLASS 
 * ------------------------------------------------------------------------
 * Copyright(c) 2007-2008 GrapeCity inc.
 */

#endregion

using System;

namespace GrapeCity.Competition.Maze.Engine
{
    #region The Time Class
    public static class Time
    {
        static Time()
        {
            long counter;
            IsHighResolution = SafeNativeMethods.QueryPerformanceCounter(out counter);

            if (IsHighResolution)
            {
                SafeNativeMethods.QueryPerformanceFrequency(out TicksPerSecond);
            }
            else
            {
                TicksPerSecond = 0x989680L;
            }

            TicksPerMillisecond = TicksPerSecond / 1000;

            startTick = Time.Ticks;
            startMilliseconds = startTick / TicksPerMillisecond;
            startSeconds = startTick / TicksPerSecond;
        }

        /// <summary>
        /// Ticks per second.
        /// </summary>
        public static readonly long TicksPerSecond;

        /// <summary>
        /// Ticks per millisecond.
        /// </summary>
        public static readonly long TicksPerMillisecond;

        /// <summary>
        /// True if a high resolution timer, other wise false.
        /// </summary>
        public static readonly bool IsHighResolution;

        private static readonly long startTick;
        private static readonly long startMilliseconds;
        private static readonly long startSeconds;

        public static long Ticks
        {
            get
            {
                if (IsHighResolution)
                {
                    long num;
                    SafeNativeMethods.QueryPerformanceCounter(out num);
                    return num;
                }
                return DateTime.UtcNow.Ticks;
            }
        }

        public static long Seconds
        {
            get { return Ticks / TicksPerSecond; }
        }

        public static long Milliseconds
        {
            get { return Ticks / TicksPerMillisecond; }
        }

        public static long FirstMilliseconds
        {
            get { return startMilliseconds; }
        }

        public static long FirstSeconds
        {
            get { return startSeconds; }
        }

        public static long FirstTick
        {
            get { return startTick; }
        }
    } 
    #endregion
}