﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using GrapeCity.Competition.Maze.Engine;
using GrapeCity.Competition.Maze.Engine.Elements;
using GrapeCity.Competition.Maze.Engine.Maps;

namespace GrapeCity.Competition.Maze.Generators
{
    [Name("Util -> Grid")]
    [Author("GrapeCity")]
    public class GridGen : IMazeMapGenerator
    {
        public void SetupRenderSupport(Control screen)
        {
        }

        public void SetupSkinSupport(IMazeSkin skin)
        {
        }

        public bool Generate(int seed, IMazeMapEditor mapEditor)
        {
            (new WallGen()).Generate(0, mapEditor);

            MapUtil map = new MapUtil(mapEditor);

            //building graph nodes.
            Position[,] graphNodes = GetGraphNodes(map.Width, map.Height);

            //hole graph nodes.
            foreach (Position node in graphNodes)
            {
                if (!map.IsExit(node) && !map.IsEntrance(node))
                {
                    map.SetElement(node, new Path(mapEditor));
                }

                Position up = Position.GetPositionByDirection(node, Direction.Up);
                if (map.IsValidPosition(up) && !map.IsExit(up) && !map.IsEntrance(up))
                {
                    map.SetElement(up, new Path(mapEditor));
                }

                Position right = Position.GetPositionByDirection(node, Direction.Right);
                if (map.IsValidPosition(right) && !map.IsExit(right) && !map.IsEntrance(right))
                {
                    map.SetElement(right, new Path(mapEditor));
                }

                Position down = Position.GetPositionByDirection(node, Direction.Down);
                if (map.IsValidPosition(down) && !map.IsExit(down) && !map.IsEntrance(down))
                {
                    map.SetElement(down, new Path(mapEditor));
                }
            }

            return true;
        }

        #region GetGraphNodes
        internal static Position[,] GetGraphNodes(int width, int height)
        {
            int graphWidth = (width % 2 == 0) ? width / 2 : (width + 1) / 2;
            int graphHeight = (height % 2 == 0) ? height / 2 : (height + 1) / 2;

            Position[,] graphNodes = new Position[graphWidth, graphHeight];

            int fix = 0;
            if (height % 2 == 0) fix = -1; //even

            for (int y = 0; y < height; y++)
            {
                for (int x = 0; x < width; x++)
                {
                    if (x % 2 == 0 && ((y + fix) % 2 == 0))
                    {
                        int gx = x / 2;
                        int gy = (y + fix) / 2;

                        graphNodes[gx, gy] = new Position(x, y);
                    }
                }
            }

            return graphNodes;
        }
        #endregion
    }
}
