﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using GrapeCity.Competition.Maze.Engine;
using GrapeCity.Competition.Maze.Engine.Elements;
using GrapeCity.Competition.Maze.Engine.Maps;

namespace GrapeCity.Competition.Maze.Generators
{
    [Name("Lv3. Ordered Maze")]
    public class OrderedMazeGen : IMazeMapGenerator
    {
        public void SetupRenderSupport(Control screen)
        {
        }

        public void SetupSkinSupport(IMazeSkin skin)
        {
        }

        public bool Generate(int seed, IMazeMapEditor map)
        {
            try
            {
                RandomUtil random = new RandomUtil(seed);
                MapUtil mapUtil = new MapUtil(map);

                mapUtil.FillWalls();

                int width = map.Width;
                int height = map.Height;

                Dictionary<Position, int> pathList = new Dictionary<Position, int>();
                Dictionary<Position, int> addPathList = new Dictionary<Position, int>();
                pathList.Add(new Position(map.Entrance), 0);

                ReCreate:
                while(true)
                {
                    foreach (Position pos in pathList.Keys)
                    {
                        Direction[] dirs = mapUtil.GetValidDirections(pos);
                        Direction ignore = Direction.None;

                        if(dirs.Length > 1)
                        {
                            ignore = random.NextDirection(dirs);
                        }

                        foreach (Direction dir in random.RandomArray(dirs))
                        {
                            if(ignore == dir) continue;
                                 
                            Position to = Position.GetPositionByDirection(pos, dir);
                            if (!mapUtil.IsEditablePosition(to)) continue;

                            if (CanPath(mapUtil, to))
                            {
                                if (!(map.GetElement(to) is Path) && !addPathList.ContainsKey(to))
                                {
                                    map.SetElement(to, new Path(map));
                                    addPathList.Add(to, 0);
                                }
                            }
                        }
                    }

                    pathList.Clear();

                    foreach (Position pos in addPathList.Keys)
                    {
                        pathList.Add(pos, 0);
                    }

                    if (addPathList.Count == 0) break;
                    addPathList.Clear();
                }
                
                pathList.Clear();

                for (int x = 0; x < width;x++ )
                {
                    for(int y=0;y<height;y++)
                    {
                        Position pos = new Position(x, y);
                        if(mapUtil.IsEditablePosition(pos) && !(map.GetElement(pos) is Path))
                        {
                            if (CanPath(mapUtil, pos))
                            {
                                map.SetElement(pos, new Path(map));
                                pathList.Add(pos, 0);
                            }
                        }
                    }
                }

                if(pathList.Count != 0) goto ReCreate;

                for (int x = 0; x < width; x++)
                {
                    for (int y = 0; y < height; y++)
                    {
                        Position pos = new Position(x, y);
                        if (mapUtil.IsEditablePosition(pos) && !(map.GetElement(pos) is Path))
                        {
                            if (CanConnect(mapUtil, pos) && random.NextBoolean())
                            {
                                map.SetElement(pos, new Path(map));
                            }
                        }
                    }
                }

                return true;
            }
            catch
            {
                return false;
            }
        }

        private static bool CanPath(MapUtil map, IPosition current)
        {
            int closePathCount = 0;
            int closeExit = 0;

            IPosition[] list = map.CloseTo(current);
            foreach (IPosition check in list)
            {
                if (!map.IsBlock(check)) closePathCount++;
                if (map.IsExit(check)) closeExit++;
            }

            if (closeExit > 0) return closePathCount > 1;
            return closePathCount == 1;
        }

        private static bool CanConnect(MapUtil map, IPosition current)
        {
            int closePathCount = 0;
            int closeExit = 0;

            IPosition[] list = map.CloseTo(current);
            foreach (IPosition check in list)
            {
                if (!map.IsBlock(check)) closePathCount++;
                if (map.IsExit(check)) closeExit++;
            }

            if (closeExit > 0) return closePathCount > 2;
            return closePathCount == 2;
        }
    }
}
