﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using GrapeCity.Competition.Maze.Engine;
using GrapeCity.Competition.Maze.Engine.Elements;
using GrapeCity.Competition.Maze.Engine.Maps;

namespace GrapeCity.Competition.Maze.Generators
{
    [Name("Lv2. Random Maze")]
    [Author("GrapeCity")]
    public class SimpleMazeMapGen : IMazeMapGenerator
    {
        public void SetupRenderSupport(Control screen)
        {
        }

        public void SetupSkinSupport(IMazeSkin skin)
        {
        }

        public bool Generate(int seed, IMazeMapEditor map)
        {
            try
            {
                Random rnd = new Random(seed);

                //make all wall
                int width = map.Width;
                int height = map.Height;

                for (int y = 0; y < height; y++)
                {
                    for (int x = 0; x < width; x++)
                    {
                        if (x == map.Entrance.X && y == map.Entrance.Y) continue;
                        if (x == map.Exit.X && y == map.Exit.Y) continue;

                        map.SetElement(x, y, new Wall(map));
                    }
                }

                Position currentPosition = new Position(map.Entrance);
                Stack<Position> path = new Stack<Position>();
                List<Direction> testDirs = new List<Direction>();

                while (true)
                {
                ReSeed:
                    Direction nextDir = (Direction)rnd.Next(0, 4);
                    Position nextPos = Position.GetPositionByDirection(currentPosition, nextDir);
                    bool isNew = false;

                    if (!testDirs.Contains(nextDir))
                    {
                        testDirs.Add(nextDir);
                        isNew = true;
                    }
                    else
                    {
                        if (testDirs.Count < 4) goto ReSeed;
                    }

                    if (CanPath(map, currentPosition, nextPos))
                    {
                        path.Push(currentPosition);
                        testDirs.Clear();

                        currentPosition = nextPos;
                        map.SetElement(currentPosition.X, currentPosition.Y, new Path(map));
                    }
                    else
                    {
                        if (!isNew)
                        {
                            if (path.Count == 0) break;
                            currentPosition = path.Pop();
                        }
                    }
                }

                currentPosition = new Position(map.Exit);
                Direction dir = rnd.Next(100) > 50 ? Direction.Right : Direction.Down;
                Position exitPos = Position.GetPositionByDirection(currentPosition, dir);
                do
                {
                    map.SetElement(exitPos, new Path(map));
                    Position lastPos = exitPos;
                    exitPos = Position.GetPositionByDirection(exitPos, dir);
                    if (!map.IsValidPosition(exitPos))
                    {
                        if (dir == Direction.Right) dir = Direction.Down;
                        else dir = Direction.Right;
                        exitPos = Position.GetPositionByDirection(lastPos, dir);
                        if (!map.IsValidPosition(exitPos)) return false;
                    }
                } while (map.GetElement(exitPos) is Wall);

                return true;
            }
            catch
            {
                return false;
            }
        }

        private static bool CanPath(IMazeMap map, IPosition from, IPosition to)
        {
            if (!map.IsValidPosition(to)) return false;
            if (map.GetElement(to) is Path) return false;
            if (map.Entrance.X == to.X && map.Entrance.Y == to.Y) return false;
            if (map.Exit.X == to.X && map.Exit.Y == to.Y) return false;
            return CanPath(map, from.X, from.Y, to.X, to.Y);
        }

        private static bool CanPath(IMazeMap map, int fromX, int fromY, int toX, int toY)
        {
            //up
            if(!IsWall(map, toX, toY - 1))
            {
                if(fromX != toX || fromY != toY - 1)
                {
                    return false;
                }
            }

            //down
            if (!IsWall(map, toX, toY + 1))
            {
                if (fromX != toX || fromY != toY + 1)
                {
                    return false;
                }
            }

            //left
            if (!IsWall(map, toX -1 , toY))
            {
                if (fromX != toX - 1 || fromY != toY)
                {
                    return false;
                }
            }

            //right
            if (!IsWall(map, toX + 1, toY))
            {
                if (fromX != toX + 1 || fromY != toY)
                {
                    return false;
                }
            }

            return true;
        }

        private static bool IsWall(IMazeMap map, int x, int y)
        {
            if (x < 0 || x >= map.Width) return true;
            if (y < 0 || y >= map.Height) return true;
            return map.GetElement(x, y) is Wall;
        }
    }
}
