﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Text;

namespace GrapeCity.Competition.Maze
{
    /// <summary>
    ///  Specifies an author name for a class.
    /// </summary>
    /// <example>
    /// <code>
    ///  [Author("GrapeCity")]
    ///  public class MyAI : IMazaAI
    ///  {
    ///    //implement your code here
    ///  }
    /// </code>
    /// </example>
    [AttributeUsageAttribute(AttributeTargets.Class, AllowMultiple=true, Inherited=false)]
    public class AuthorAttribute : Attribute
    {
        private readonly string m_author;

        /// <summary>
        ///  Create a new instance of <see cref="AuthorAttribute"/> object.
        /// </summary>
        /// <param name="author">A <b>string</b> value indicates the author name.</param>
        public AuthorAttribute(string author)
        {
            m_author = author;
        }

        /// <summary>
        ///  Gets the author name of this AI.
        /// </summary>
        /// <value>A <b>string</b> value indicates the author name.</value>
        public string Author
        {
            get { return m_author; }
        }

        /// <summary>
        /// Gets the author name from a type.
        /// </summary>
        /// <param name="type">The type contains one or more <see cref="AuthorAttribute"/>.</param>
        /// <returns>A <b>string</b> value indicates the all author name.</returns>
        public static string GetName(Type type)
        {
            Trace.Assert(type != null);

            object[] attributeArray = type.GetCustomAttributes(typeof(AuthorAttribute), false);
            if (attributeArray != null && attributeArray.Length > 0)
            {
                StringBuilder builder = new StringBuilder();
                bool isFirstAuthor = true;

                foreach (AuthorAttribute attribute in attributeArray)
                {
                    if(attribute != null)
                    {
                        if (isFirstAuthor)
                        {
                            isFirstAuthor = false;
                        }
                        else
                        {
                            builder.Append("; ");
                        }
                        builder.Append(attribute.Author);
                    }
                }

                return builder.ToString();
            }
            return "Unknown Author";
        }
    }
}
