﻿using System;

namespace GrapeCity.Competition.Maze
{
    /// <summary>
    ///  Represents the AI.
    /// </summary>
    /// <example>
    /// <code>
    /// [Name("Random Walker")]
    /// [Author("GrapeCity")]
    /// public class RandomAI : IMazeAI
    /// {
    ///     private bool terminated = false;
    ///     public void Think(IMaze maze, IExplorer explorer)
    ///     {
    ///         try
    ///         {
    ///             Random rnd = new Random();
    ///             while (!terminated)
    ///             {
    ///                 explorer.Move((Direction)rnd.Next(4));
    /// 
    ///                 if (explorer.Current == PositionInfo.Exit)
    ///                 {
    ///                     explorer.Say("Too easy, I get out!!", null);
    ///                     break;
    ///                 }
    ///             }
    ///         }
    ///         finally
    ///         {
    ///         }
    ///     }
    ///
    ///     public void Hear(string name, string message, object information)
    ///     {
    ///     }
    ///
    ///     public void Initialize()
    ///     {
    ///     }
    ///
    ///     public void Terminate()
    ///     {
    ///         terminated = true;
    ///     }
    /// }
    /// </code>
    /// </example>
    public interface IMazeAI
    {
        /// <summary>
        ///  Initializes the <see cref="IMazeAI"/> object.
        /// </summary>
        void Initialize();

        /// <summary>
        /// Terminates the <see cref="IMazeAI"/> object.
        /// </summary>
        void Terminate();

        /// <summary>
        ///  Move the explorer to the exit of the maze. 
        /// </summary>
        /// <param name="maze">An <b>IMaze</b> object indicates the maze.</param>
        /// <param name="explorer">An <b>IExplorer</b> object indicates the explorer.</param>
        void Think(IMaze maze, IExplorer explorer);

        /// <summary>
        ///  Occurs when the explorer hear any message.
        /// </summary>
        /// <param name="name">A <b>string</b> value indicates the explorer name of sender.</param>
        /// <param name="message">A <b>string</b> value indicates the message.</param>
        /// <param name="information">An <see cref="Object"/> indicates the extra information.</param>
        void Hear(string name, string message, object information);
    }
}
