﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Text;

namespace GrapeCity.Competition.Maze
{
    /// <summary>
    ///  Specifies a name for a class.
    /// </summary>
    /// <example>
    /// <code>
    ///  [Name("AutoBot")]
    ///  public class MyAI : IMazaAI
    ///  {
    ///    //implement your code here
    ///  }
    /// </code>
    /// </example>
    [AttributeUsageAttribute(AttributeTargets.Class, AllowMultiple = false, Inherited = false)]
    public class NameAttribute : Attribute
    {
        private readonly string m_name;

        /// <summary>
        ///  Create a new instance of <see cref="NameAttribute"/> object.
        /// </summary>
        /// <param name="name">A <b>string</b> value indicates the AI name.</param>
        public NameAttribute(string name)
        {
            m_name = name;
        }

        /// <summary>
        ///  Gets the AI name.
        /// </summary>
        /// <value>
        ///  A <b>string</b> value indicates the AI name.
        /// </value>
        public string Name
        {
            get { return m_name; }
        }

        /// <summary>
        /// Gets the AI name from a type.
        /// </summary>
        /// <param name="type">The type contains a <see cref="NameAttribute"/>.</param>
        /// <returns>A <b>string</b> value indicates the AI name.</returns>
        public static string GetName(Type type)
        {
            Trace.Assert(type != null);

            object[] attributeArray = type.GetCustomAttributes(typeof (NameAttribute), false);
            if(attributeArray != null && attributeArray.Length > 0 && attributeArray[0] is NameAttribute)
            {
                return ((NameAttribute) attributeArray[0]).Name;
            }
            return "Undefined Name";
        }
    }
}
