﻿using System;
using System.Runtime.InteropServices;

namespace GrapeCity.Competition.Maze
{
    /// <summary>
    ///  Represents the position in maze map.
    /// </summary>
    [Serializable]
    public struct Position : IPosition
    {
        /// <summary>
        ///   Gets the position indicates the zero.
        /// </summary>
        public static readonly Position Zero;


        int pointX;
        int pointY;

        ///<summary>
        /// Initializes a new instance of the <see cref="Position"/> class with the specified coordinates.
        ///</summary>
        ///<param name="x">The horizontal position of the <see cref="Position"/>. </param>
        ///<param name="y">The vertical position of the <see cref="Position"/>.</param>
        public Position(int x, int y)
        {
            pointX = x;
            pointY = y;
        }

        ///<summary>
        /// Initializes a new instance of the <see cref="Position"/> class from a <see cref="IPosition"/>.
        ///</summary>
        ///<param name="p">A <see cref="IPosition"/> that specifies the coordinates for the new <see cref="Position"/>. </param>
        public Position(IPosition p)
            : this(p.X, p.Y)
        {
        }

        /// <summary>
        /// Gets or sets the x-coordinate of this <see cref="Position"/>.
        /// </summary>
        public int X
        {
            get { return pointX; }
            set { pointX = value; }
        }

        /// <summary>
        /// Gets or sets the y-coordinate of this <see cref="Position"/>.
        /// </summary>
        public int Y
        {
            get { return pointY; }
            set { pointY = value; }
        }

        /// <summary>
        ///  Converts this <see cref="Position"/> to a human-readable string.
        /// </summary>
        /// <returns>
        ///  A <see cref="String"/> containing a human-readable string.
        /// </returns>
        public override string ToString()
        {
            return string.Format("({0}, {1})", pointX, pointY);
        }

        /// <summary>
        /// Specifies whether this <see cref="Position"/> contains the same coordinates as the specified <see cref="Object"/>.
        /// </summary>
        /// <param name="o">The <see cref="Object"/> to test. </param>
        /// <returns><b>true</b> if obj is a <see cref="Position"/> and has the same coordinates as this <see cref="Position"/>.</returns>
        public override bool Equals(object o)
        {
            if ((o != null) && (o is IPosition))
            {
                IPosition pointd1 = (IPosition) o;
                if (pointX == pointd1.X)
                {
                    return (pointY == pointd1.Y);
                }
            }
            return false;
        }

        /// <summary>
        /// Returns a hash code for this <see cref="Position"/>.
        /// </summary>
        /// <returns>An integer value that specifies a hash value for this <see cref="Position"/>.</returns>
        public override int GetHashCode()
        {
            int x = pointX + 1;
            int y = pointY + 1;

            return x + y * 1024;
        }

        //public static bool operator ==(Position l, Position r)
        //{
        //    if (l.pointX == r.pointX)
        //    {
        //        return (l.pointY == r.pointY);
        //    }
        //    return false;
        //}

        //public static bool operator !=(Position l, Position r)
        //{
        //    if (l.pointX == r.pointX)
        //    {
        //        return (l.pointY != r.pointY);
        //    }
        //    return true;
        //}

        //public static bool operator ==(Position l, IPosition r)
        //{
        //    if (l.pointX == r.X)
        //    {
        //        return (l.pointY == r.Y);
        //    }
        //    return false;
        //}

        //public static bool operator !=(Position l, IPosition r)
        //{
        //    if (l.pointX == r.X)
        //    {
        //        return (l.pointY != r.Y);
        //    }
        //    return true;
        //}

        //public static bool operator >(Position l, Position r)
        //{
        //    if (l.pointX > r.pointX)
        //    {
        //        return (l.pointY > r.pointY);
        //    }
        //    return false;
        //}

        //public static bool operator >(Position l, IPosition r)
        //{
        //    if (l.pointX > r.X)
        //    {
        //        return (l.pointY > r.Y);
        //    }
        //    return false;
        //}

        //public static bool operator <(Position l, Position r)
        //{
        //    if (l.pointX < r.pointX)
        //    {
        //        return (l.pointY < r.pointY);
        //    }
        //    return false;
        //}

        //public static bool operator <(Position l, IPosition r)
        //{
        //    if (l.pointX < r.X)
        //    {
        //        return (l.pointY < r.Y);
        //    }
        //    return false;
        //}

        //public static bool operator >=(Position l, Position r)
        //{
        //    if (l.pointX >= r.pointX)
        //    {
        //        return (l.pointY >= r.pointY);
        //    }
        //    return false;
        //}

        //public static bool operator >=(Position l, IPosition r)
        //{
        //    if (l.pointX >= r.X)
        //    {
        //        return (l.pointY >= r.Y);
        //    }
        //    return false;
        //}

        //public static bool operator <=(Position l, Position r)
        //{
        //    if (l.pointX <= r.pointX)
        //    {
        //        return (l.pointY <= r.pointY);
        //    }
        //    return false;
        //}


        //public static bool operator <=(Position l, IPosition r)
        //{
        //    if (l.pointX <= r.X)
        //    {
        //        return (l.pointY <= r.Y);
        //    }
        //    return false;
        //}

        static Position()
        {
            Zero = new Position(0, 0);
        }

        /////<summary>
        /////</summary>
        /////<param name="value"></param>
        /////<returns></returns>
        //public static Position Parse(string value)
        //{
        //    int num1 = value.IndexOf('(');
        //    int num2 = value.IndexOf(',', num1 + 1);
        //    string text1 = value.Substring(num1 + 1, num2 - (num1 + 1)).Trim();
        //    num1 = num2;
        //    num2 = value.IndexOf(')', num1 + 1);
        //    string text2 = value.Substring(num1 + 1, num2 - (num1 + 1)).Trim();
        //    return new Position(Convert.ToInt32(text1), Convert.ToInt32(text2));
        //}

        /// <summary>
        ///  Gets the position for the specified direction of the source position.
        /// </summary>
        /// <param name="source">The source position.</param>
        /// <param name="direction">The direction.</param>
        /// <returns>The position at the direction.</returns>
        public static Position GetPositionByDirection(IPosition source, Direction direction)
        {
            switch (direction)
            {
                case Direction.Right:
                    return new Position(source.X + 1, source.Y);
                case Direction.Left:
                    return new Position(source.X - 1, source.Y);
                case Direction.Up:
                    return new Position(source.X, source.Y - 1);
                case Direction.Down:
                    return new Position(source.X, source.Y + 1);
                default:
                    return new Position(source);
            }
        }

        /// <summary>
        /// Compares two <see cref="Position"/> objects. The result specifies whether the values of the X and Y properties of the two <see cref="Position"/> objects are equal.
        /// </summary>
        /// <param name="left">A <see cref="Position"/> to compare. </param>
        /// <param name="right">A <see cref="Position"/> to compare. </param>
        /// <returns><b>true</b> if the <seealso cref="X"/> and <seealso cref="X"/> values of left and right are equal; otherwise, <b>false</b>.</returns>
        public static bool Equality(IPosition left, IPosition right)
        {
            return left.X == right.X && left.Y == right.Y;
        }
    }
}