﻿using System;
using System.Collections.Generic;
using System.Drawing;
using GrapeCity.Competition.Maze.Engine;
using GrapeCity.Competition.Maze.Engine.Elements;
using GrapeCity.Competition.Maze.Engine.Explorers;

namespace GrapeCity.Competition.Maze.EmbeddedEngine
{
    internal class MazeSkin : IMazeSkin
    {
        internal const int Color_Explorer = 0;
        internal const int Color_Entrance = 1;
        internal const int Color_Exit = 2;
        internal const int Color_Path = 3;
        internal const int Color_Wall = 4;
        internal const int Color_DoorOpen = 5;
        internal const int Color_Trigger = 6;
        internal const int Color_DoorClose = 7;
        internal const int Color_Transport = 8;

        private const int Color_Count = 9;

        private readonly List<Color> elementColors = new List<Color>(Color_Count);
        private int m_blockSize = 32;

        internal MazeSkin()
        {
            ResetColor();
        }

        internal void ResetColor()
        {
            elementColors.Clear();

            elementColors.Add(Color.Black);
            elementColors.Add(Color.Black);
            elementColors.Add(Color.Black);
            elementColors.Add(Color.Black);
            elementColors.Add(Color.Black);
            elementColors.Add(Color.Black);
            elementColors.Add(Color.Black);
            elementColors.Add(Color.Black);
            elementColors.Add(Color.Black);

            elementColors[Color_Explorer] = Color.Red;
            elementColors[Color_Entrance] = Color.White;
            elementColors[Color_Wall] = Color.DarkOliveGreen;
            elementColors[Color_Exit] = Color.DodgerBlue;
            elementColors[Color_Trigger] = Color.DimGray;
            elementColors[Color_Path] = Color.White;
            elementColors[Color_DoorOpen] = Color.LightCyan;
            elementColors[Color_DoorClose] = Color.Firebrick;
            elementColors[Color_Transport] = Color.OldLace;
        }

        internal void ThemeDarkBlue()
        {
            elementColors[Color_Explorer] = Color.Yellow;
            elementColors[Color_Entrance] = Color.DarkBlue;
            elementColors[Color_Wall] = Color.Black;
            elementColors[Color_Exit] = Color.DodgerBlue;
            elementColors[Color_Trigger] = Color.DimGray;
            elementColors[Color_Path] = Color.DarkBlue;
            elementColors[Color_DoorOpen] = Color.LightCyan;
            elementColors[Color_DoorClose] = Color.Firebrick;
            elementColors[Color_Transport] = Color.OldLace;
        }

        internal Color this[int code]
        {
            get { return elementColors[code]; }
            set { elementColors[code] = value; }
        }

        internal List<int> SkinColors
        {
            get
            {
                List<int> colorList = new List<int>();
                foreach (Color color in elementColors)
                {
                    colorList.Add(color.ToArgb());
                }

                return colorList;
            }
            set
            {
                if (value == null) return;

                for (int idx = 0; idx < Color_Count; idx++)
                {
                    if(idx<value.Count)
                    {
                        elementColors[idx] = Color.FromArgb(value[idx]);
                    }
                }
            }
        }

        public int BlockSize
        {
            get { return m_blockSize; }
            set { m_blockSize = value; }
        }

        public virtual void PaintElement(Graphics g, int x, int y, MazeElement element)
        {
            PaintElement(g, x, y, m_blockSize, element);
        }

        public void PaintElement(Graphics g, int x, int y, int size, MazeElement element)
        {
            if (element == null) return;

            int elementCode = (int)element.ElementInfo;

            if (element is Door)
            {
                Door door = element as Door;
                elementCode = door.IsLocked ? Color_DoorClose : Color_DoorOpen;
            }
            else if (element is Transport)
            {
                elementCode = Color_Transport;
            }

            Color elementColor = elementColors[elementCode];
            Brush brush = new SolidBrush(elementColor);
            g.FillRectangle(brush, x * size, y * size, size, size);
        }

        public virtual void PaintExplorer(Graphics g, MazeExplorer e)
        {
            Color explorerColor = elementColors[Color_Explorer];
            Brush brush = new SolidBrush(explorerColor);
            g.FillRectangle(brush, e.X * m_blockSize, e.Y * m_blockSize, m_blockSize, m_blockSize);
        }

        public void PaintFootPrint(Graphics g, int x, int y)
        {
            Color explorerColor = elementColors[Color_Explorer];
            Brush brush = new SolidBrush(Color.FromArgb(50, explorerColor));
            g.FillRectangle(brush, x * m_blockSize, y * m_blockSize, m_blockSize, m_blockSize);
        }
    }
}
