﻿using System;
using System.Collections.Generic;
using System.Reflection;
using System.Text;

namespace GrapeCity.Competition.Maze
{
    internal class MazePlayer
    {
        private string name;
        private string mazeAiVersion;
        private string mazeAiName;
        private string mazeAiAuthor;
        private readonly Type mazeAiType;

        internal MazePlayer(string name, Type mazeAiType)
        {
            if (mazeAiType == null) throw new ArgumentNullException("mazeAiType");

            this.name = name;
            this.mazeAiType = mazeAiType;

            LoadAssembly(mazeAiType);
        }

        #region Load Assembly
        private void LoadAssembly(Type mazeAi)
        {
            this.mazeAiName = NameAttribute.GetName(mazeAi);
            this.mazeAiAuthor = AuthorAttribute.GetName(mazeAi);
            this.mazeAiVersion = mazeAi.Assembly.GetName().Version.ToString();
        } 
        #endregion

        /// <summary>
        /// Gets or sets the player name.
        /// </summary>
        internal string Name
        {
            get { return name; }
            set { name = value; }
        }

        /// <summary>
        /// Gets version of the maze ai.
        /// </summary>
        internal string Version
        {
            get { return mazeAiVersion; }
        }

        /// <summary>
        /// Gets ai name of the maze ai.
        /// </summary>
        internal string AIName
        {
            get { return mazeAiName; }
        }

        /// <summary>
        /// Gets author name of the maze ai.
        /// </summary>
        internal string AIAuthor
        {
            get { return mazeAiAuthor; }
        }

        /// <summary>
        /// Gets the ai type.
        /// </summary>
        internal Type AIType
        {
            get { return mazeAiType; }
        }
    }

}
