﻿using System;
using System.Collections.Generic;
using System.Text;
using GrapeCity.Competition.Maze.Engine;
using GrapeCity.Competition.Maze.Engine.Explorers;

namespace GrapeCity.Competition.Maze
{
    internal class MazeStatus
    {
        private readonly string filename;
        private readonly string name;
        private readonly int width;
        private readonly int height;
        private readonly int reachBlock;
        private readonly int totalBlock;
        private readonly MazePlayerStatus[] players;
        private readonly Position exit;

        private readonly int looks;
        private readonly int steps;
        private readonly int blocks;
        private readonly int maxstep = 0;
        private readonly int minstep = int.MaxValue;
        private readonly int avgstep = 0;

        internal MazeStatus(string mazeFilename)
        {
            this.name = "Untitled Map";
            this.filename = mazeFilename;
            this.totalBlock = 400;
            this.reachBlock = 0;
            this.width = 20;
            this.height = 20;
        }

        internal MazeStatus(string name, int width, int height, string filename, IPosition exit, int reachBlock, int totalBlock, MazeExplorer[] explorers)
        {
            this.name = name;
            this.totalBlock = totalBlock;
            this.reachBlock = reachBlock;
            this.filename = filename;
            this.height = height;
            this.width = width;
            this.exit = new Position(exit);

            List<MazePlayerStatus> status = new List<MazePlayerStatus>();
            foreach (MazeExplorer explorer in explorers)
            {
                MazePlayerStatus mps = new MazePlayerStatus(explorer);
                looks = Looks + mps.Looks;
                steps = Steps + mps.Steps;
                blocks = Blocks + mps.Blocks;
                maxstep = Math.Max(Maxstep, mps.Steps);
                minstep = Math.Min(Minstep, mps.Steps);
                status.Add(mps);
            }

            players = status.ToArray();
            if (status.Count >0 )
                avgstep = Steps/status.Count;
            else
            {
                avgstep = 0;
                minstep = 0;
            }
            
        }

        internal string Name
        {
            get { return name; }
        }

        internal int Width
        {
            get { return width; }
        }

        internal int Height
        {
            get { return height; }
        }

        internal string Filename
        {
            get { return filename; }
        }

        internal int ReachBlock
        {
            get { return reachBlock; }
        }

        internal int TotalBlock
        {
            get { return totalBlock; }
        }

        internal double Complete
        {
            get { return (double)reachBlock/totalBlock; }
        }

        internal MazePlayerStatus[] Players
        {
            get { return players; }
        }

        internal int Looks
        {
            get { return looks; }
        }

        internal int Steps
        {
            get { return steps; }
        }

        internal int Blocks
        {
            get { return blocks; }
        }

        internal int Maxstep
        {
            get { return maxstep; }
        }

        internal int Minstep
        {
            get { return minstep; }
        }

        internal int Avgstep
        {
            get { return avgstep; }
        }

        internal Position Exit
        {
            get { return exit; }
        }
    }
}
