﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Reflection;
using System.Text;
using System.Windows.Forms;

namespace GrapeCity.Competition.Maze.UI
{
    internal partial class AssemblyLibraryDialog : Form
    {
        private readonly List<string> assemblyList;

        internal AssemblyLibraryDialog(List<string> assemblyList)
        {
            InitializeComponent();
            this.assemblyList = assemblyList;

            if(assemblyList == null || assemblyList.Count == 0)
            {
                buttonImport_Click(null, null);
            }
        }

        private void buttonImport_Click(object sender, EventArgs e)
        {
            if(openAssemblyDialog.ShowDialog(this) == DialogResult.OK)
            {
                if(openAssemblyDialog.FileNames != null)
                {
                    foreach (string fileName in openAssemblyDialog.FileNames)
                    {
                        if(File.Exists(fileName))
                        {
                            if(!this.assemblyList.Contains(fileName))
                            {
                                this.assemblyList.Add(fileName);
                            }
                        }
                    }

                    RefreshAssembly();
                }
            }
        }

        private void RefreshAssembly()
        {
            listView.Items.Clear();

            if (assemblyList == null) return;

            foreach (string filename in assemblyList)
            {
                ListViewItem item = new ListViewItem();
                item.Tag = filename;

                try
                {
                    Assembly temp = Assembly.LoadFile(filename);
                    item.Text = temp.GetName().Name;
                    item.SubItems.Add(temp.GetName().Version.ToString());
                    item.SubItems.Add(filename);
                }
                catch (Exception e)
                {
                    item.Text = e.Message;
                    item.SubItems.Add("Unknown");
                    item.SubItems.Add(filename);
                }

                listView.Items.Add(item);
            }

            listView.AutoResizeColumns(ColumnHeaderAutoResizeStyle.HeaderSize);
        }

        private void AssemblyLibraryDialog_Load(object sender, EventArgs e)
        {
            RefreshAssembly();
        }

        private void buttonRemove_Click(object sender, EventArgs e)
        {
            if(listView.SelectedItems != null && listView.SelectedItems.Count >  0)
            {
                foreach (ListViewItem item in listView.SelectedItems)
                {
                    string fileName = item.Tag as string;
                    if(fileName != null)
                    {
                        if(assemblyList.Contains(fileName))
                        {
                            assemblyList.Remove(fileName);
                        }
                    }
                }

                RefreshAssembly();
            }
        }

        private void buttonClose_Click(object sender, EventArgs e)
        {
            this.Close();
        }
    }
}
