﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace GrapeCity.Competition.Maze.UI
{
    internal partial class CreateMazeDialog : Form
    {
        internal CreateMazeDialog()
        {
            InitializeComponent();
        }

        private void buttonCancel_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
            Close();
        }

        private void buttonCreate_Click(object sender, EventArgs e)
        {
            if(string.IsNullOrEmpty(mazeName.Text))
            {
                errorProvider1.SetError(mazeName, "The maze name is empty.");
                mazeName.Focus();
            }
            else if(!IsValidEE())
            {
                errorProvider1.SetError(mazeName, "The maze entrance or exit is not valid.");
            }
            else
            {
                DialogResult = DialogResult.OK;
                Close();
            }
        }

        private bool IsValidEE()
        {
            int height = (int) mazeHeight.Value;
            if (height < 2 || height > 500) return false;

            int y = (int)entranceY.Value;
            if(y <0 || y>= height) return false;

            y = (int)exitY.Value;
            if (y < 0 || y >= height) return false;

            int width = (int)mazeWidth.Value;
            if (width < 2 || width > 500) return false;

            int x = (int)entranceX.Value;
            if (x < 0 || x >= width) return false;

            x = (int)exitX.Value;
            if (x < 0 || x >= width) return false;

            if ((int)entranceX.Value == (int)exitX.Value && (int)entranceY.Value == (int)exitY.Value) return false;

            return true;
        }


        internal string MazeName
        {
            get { return mazeName.Text; }
        }

        internal int MazeHeight
        {
            get { return (int)mazeHeight.Value; }
        }

        internal int MazeWidth
        {
            get { return (int)mazeWidth.Value; }
        }

        internal Position Entrance
        {
            get { return new Position((int)entranceX.Value, (int)entranceY.Value); }
        }

        internal Position Exit
        {
            get { return new Position((int)exitX.Value, (int)exitY.Value); }
        }

        private void mazeWidth_ValueChanged(object sender, EventArgs e)
        {
            entranceX.Value = (int)mazeWidth.Value - 1;
        }

        private void mazeHeight_ValueChanged(object sender, EventArgs e)
        {
            entranceY.Value = (int)mazeHeight.Value - 1;
        }

        private void mazeWidth_KeyUp(object sender, KeyEventArgs e)
        {
            entranceX.Value = (int)mazeWidth.Value - 1;
        }

        private void mazeHeight_KeyUp(object sender, KeyEventArgs e)
        {
            entranceY.Value = (int)mazeHeight.Value - 1;
        }
    }
}
