﻿using System;
using System.Collections.Generic;
using System.Reflection;
using System.Text;

namespace GrapeCity.Competition.Maze.Utils
{
    internal class ListComparer<T> : IComparer<T>
    {
        private readonly string propertyName;

        private ListComparer(string PropertyName)
        {
            propertyName = PropertyName;
        }
        
        #region IComparer<T> Members

        public int Compare(T x, T y)
        {
            PropertyInfo property = typeof(T).GetProperty(propertyName, BindingFlags.Public | BindingFlags.Instance | BindingFlags.NonPublic);
            if (property.PropertyType == typeof(Int16))
            {
                int xNumber = 0;
                int yNumber = 0;
                if (property.GetValue(x, null) != null)
                {
                    xNumber = Convert.ToInt16(property.GetValue(x, null).ToString());
                }
                if (property.GetValue(y, null) != null)
                {
                    yNumber = Convert.ToInt16(property.GetValue(y, null).ToString());
                }
                return xNumber.CompareTo(yNumber);
            }
            if (property.PropertyType == typeof(Int32))
            {
                int xNumber = 0;
                int yNumber = 0;
                if (property.GetValue(x, null) != null)
                {
                    xNumber = Convert.ToInt32(property.GetValue(x, null).ToString());
                }
                if (property.GetValue(y, null) != null)
                {
                    yNumber = Convert.ToInt32(property.GetValue(y, null).ToString());
                }
                return xNumber.CompareTo(yNumber);
            }
            if (property.PropertyType == typeof(Double))
            {
                double xNumber = 0;
                double yNumber = 0;
                if (property.GetValue(x, null) != null)
                {
                    xNumber = Convert.ToDouble(property.GetValue(x, null).ToString());
                }
                if (property.GetValue(y, null) != null)
                {
                    yNumber = Convert.ToDouble(property.GetValue(y, null).ToString());
                }
                return xNumber.CompareTo(yNumber);
            }
            if (property.PropertyType == typeof(DateTime))
            {
                DateTime xTime = DateTime.Now;
                DateTime yTime = DateTime.Now;
                if (property.GetValue(x, null) != null)
                {
                    xTime = Convert.ToDateTime(property.GetValue(x, null).ToString());
                }
                if (property.GetValue(y, null) != null)
                {
                    yTime = Convert.ToDateTime(property.GetValue(y, null).ToString());
                }
                return xTime.CompareTo(yTime);
            }
            if ((property.PropertyType == typeof(String)) || (property.PropertyType == typeof(Boolean)))
            {
                string xText = string.Empty;
                string yText = string.Empty;
                if (property.GetValue(x, null) != null)
                {
                    xText = property.GetValue(x, null).ToString();
                }
                if (property.GetValue(y, null) != null)
                {
                    yText = property.GetValue(y, null).ToString();
                }
                return xText.CompareTo(yText);
            }
            return 0;

        }

        #endregion

        internal static IComparer<T> FromProperty(string propertyName)
        {
            return new ListComparer<T>(propertyName);
        }
    }

}
